#!/bin/bash

echo
echo "Scan and slice each scanned page into strips"

# SETTINGS
#
SCAN_RESOLUTION="900"
# image format INCLUDING DOT
STRIP_IMG_EXT=".pgm"
# Contact Sheet images horizontally
STRIPS_PER_PAGE_X="30"
# Contact Sheet images vertically
STRIPS_PER_PAGE_Y="1"
# imagemagick crop settings
STRIP_CROP_PARAMS="${STRIPS_PER_PAGE_X}x${STRIPS_PER_PAGE_Y}+1+1@ +repage"

# ################################################3

read -e -p "Path to store scanned images: " -i "./scannedStrips" DEST_FRAMES_PATH
read -e -p "DPI to scan at: " -i "$SCAN_RESOLUTION" SCAN_RESOLUTION

# clean up old slices
echo 
echo removing any previously scanned images
rm -r "$DEST_FRAMES_PATH"
mkdir -p $DEST_FRAMES_PATH

# begin scanning
let FRAMES_PER_PAGE=$(( $STRIPS_PER_PAGE_X * $STRIPS_PER_PAGE_Y ))
PAGE_COUNT=0
MORE_PAGES=y
while [ "$MORE_PAGES" = "y" ]; do
  FIRST_FRAMENUM=$(( PAGE_COUNT * FRAMES_PER_PAGE ))
  FIRST_FRAMENUM=$(( FIRST_FRAMENUM + 1 ))
  LAST_FRAMENUM=$(( (PAGE_COUNT + 1) * FRAMES_PER_PAGE ))

  echo 
  echo "scanning.... then...."
  echo "extracting $FRAMES_PER_PAGE frames to $DEST_FRAMES_PATH"
  echo "this is page number $(( PAGE_COUNT + 1 ))"
  echo "frames numbered $FIRST_FRAMENUM to $LAST_FRAMENUM"

  # full a4 is -x 210 -y 297
  scanimage -l 2 -t 1 -x 203 -y 292 \
  --mode Gray \
  --calibration-cache=yes \
  --resolution $SCAN_RESOLUTION \
  | convert - -monitor \
  -crop $STRIP_CROP_PARAMS \
  -scene $FIRST_FRAMENUM \
  "${DEST_FRAMES_PATH}/%010d${STRIP_IMG_EXT}"

  (( PAGE_COUNT += 1 ))

echo "Scanned $PAGE_COUNT so far"
read -e -p "More pages?: " -i "y" MORE_PAGES
done

echo
echo Done
