import Image,ImageDraw
import math 
import wave
import struct
import sys
import getopt
import os

"""
produce an audio file from an image 
one image for one second of audio
i.e. number of y pixels represents the sample rate

sum intensity of each row of x pixels represents a sample value
e.g. a row of x pixels which are 
  1. totally black represents a full negative deflection
  2. totally white represents full positive deflection
  3. exactly half white,half black represents zero deflection
"""

global glob_gain
glob_gain = 1

# avoid clicks by trimming top/bottom pixels
global glob_trimTop
global glob_trimBtm
glob_trimTop = 0
glob_trimBtm = 0

## write image data into wav file
def appendToWav(img, wav):

  imgData = img.load()
  xAvg = 0
  for y in range(0+glob_trimTop, imgHeight-glob_trimBtm):
     ## iterate over y-axis, compute average over the x-axis
     xSum = 0;
     for x in range(0, imgWidth):
       xSum += imgData[x,y]

     xAvg = int(xSum / imgWidth)

     # apply gain
     if glob_gain != 1:
       xAvg = xAvg - 128
       xAvg = xAvg * glob_gain
       xAvg = xAvg + 128

     # TODO - highpass filter the signal
     #xAvg = 128 + (128*highpass( xAvg/255.0 + 0.5 ))
     #xAvg = highpass(10*xAvg)/5
     #xAvg -= overallAverage

     # clip
     if xAvg < 0: xAvg = 0
     if xAvg > 255: xAvg = 255

     ## push our computed x-axis-average onto the wavData array
     ## we must first pack into a struct
     ## mode B is unsigned 8-bit (0-255. 128 quiescent)
     wav.writeframesraw( struct.pack("B", xAvg) )
  
if __name__ == "__main__":
  try:
    opts, args = getopt.getopt(sys.argv[1:], "w:m:g:")
  except getopt.GetoptError:
    print("error parsing commandline arguments")
    print("-m <image directory> -w <output wav name> [-g gain]")
    sys.exit(2)
  
  imageDirectory = "stripsPNG"
  wavFilename = "reassembled.wav"
  for o, a in opts:
    if o == "-m":
      imageDirectory = a
    if o == "-w":
      wavFilename = a
    if o == "-g":
      glob_gain = int(a)

  print("opening %s for writing" % wavFilename)
  wav = wave.open(wavFilename, "w")
  wav.setnchannels(1)
  wav.setsampwidth(1)
  wav.setcomptype("NONE","") ## uncompressed

  # before we can set the sample rate
  # we must scan the resolution of all images
  # -- each image represents a second of audio so we can
  #    count the total number of pixels and divide by the number of images
  print("looking in %s directory for images..." % imageDirectory)
  print("counting images, calculating average imgHeight, thus samplerate")
  numProcessedImages = 0
  imgHeightSum = 0
  for root,dirs,files in os.walk(imageDirectory):
    if root == imageDirectory: # do not recurse into subdirs of the imagedir
      for imageFilename in files:
        imageFilename = imageDirectory + "/" + imageFilename
        img = Image.open(imageFilename)
        imgWidth, imgHeight = img.size
        imgWidth, imgHeight = img.size
        imgHeightSum += imgHeight
        numProcessedImages += 1
  wav.setframerate(imgHeightSum/numProcessedImages)

  # now process the image data
  print("now converting images to sound")
  for root,dirs,files in os.walk(imageDirectory):
    if root == imageDirectory: # do not recurse into subdirs of the imagedir
      for imageFilename in sorted(files):
        imageFilePath = imageDirectory + "/" + imageFilename
        print("opening %s for reading" % imageFilename)
        img = Image.open(imageFilePath).convert("L") # greyscale 8-bits per pixel
        appendToWav(img, wav)
  
  # patch up the header with the number of written frames and close the file
  wav.writeframes('')
  wav.close()
  print("finished writing wav")
      
