import Image,ImageDraw
import math 
import wave
import struct
import sys
import getopt

"""
produce an image where entire image represents a second of audio
i.e. the number of y pixels represents the sample rate

sum intensity of each row of x pixels represents a sample value
e.g. a row of x pixels which are 
  1. totally black represents a full negative deflection
  2. totally white represents full positive deflection
  3. exactly half white,half black represents zero deflection

example usage:

mkdir -p strips
python makeImageStripsFromWav.py -i strips/_.png -w input.wav
"""
 
##plotOneSecondFromWav
#  wav = wave object from wave.open(wavFilename, "r")
#  img = PIL img object
#  draw = PIL draw object
def plotOneSecondFromWav(wav, img, draw):

  if debug:
    print("frames: %d" % wav.getnframes() )
    print("chans: %d" % wav.getnchannels() )
    print("srate: %d" % wav.getframerate() )
    print("byt/s: %d" % wav.getsampwidth() )

  # precompute scaling factor
  # to between 0 and right edge (assuming input is 0-255 128 quiescent)  
  imgWidth, imgHeight = img.size
  multiplierForRightEdge = imgWidth / 255.0
  
  # let's plot the image now
  imgYPos = 0
  wavFrames = wav.readframes( wav.getframerate() )
  i = 0;
  for frame in wavFrames:
    i += 1
    
    ## unpack gives tuple even if asked for 1 value    
    intSample = struct.unpack('B', frame)[0]
    
    ## apply gain (shifting to compensate for quiescent offset)
    ## and clip
    intSample -= 127
    intSample *= glob_gain
    intSample += 127
    if (intSample>255): intSample = 255
    if (intSample<0): intSample = 0

    # scale to between 0 and right edge (assuming input is 0-255 128 quiescent)
    plotVal = intSample * multiplierForRightEdge
  
    # plot
    draw.line( (0, imgYPos, plotVal, imgYPos), (0) )
    draw.line( (plotVal, imgYPos, imgWidth, imgYPos), (255) ) 
    imgYPos += 1

    
def wavLengthInSeconds(wav):
  wavFrames = wav.getnframes()
  wavRate = wav.getframerate()

  wavLengthInSeconds = wavFrames / wavRate
  print("length in seconds: %d" % wavLengthInSeconds)

  return wavLengthInSeconds

if __name__ == "__main__":
  global debug
  global glob_imgWidth
  global glob_gain

  glob_imgWidth = 200
  glob_gain = 1
  debug = False

  try:
    opts, args = getopt.getopt(sys.argv[1:], "i:w:d:g:")
  except getopt.GetoptError:
    print("error parsing commandline arguments") 
    sys.exit(2)
  
  imageFilePrefix = "output"
  wavFilename = "input.wav"
  for o, a in opts:
    if o == "-i":
      imageFilename = a
    if o == "-w":
      wavFilename = a
    if o == "-d":
      glob_imgWidth = int(a)
    if o == "-g":
      glob_gain = int(a)

  ## define our wav object
  wav = wave.open(wavFilename, "r") 

  ## abort if sample width is not 1 byte per sample
  if wav.getsampwidth() != 1 or wav.getnchannels() != 1:
      print("sound file must be mono pcm_u8 unsigned 8-bits per sample");
      sys.exit(1)

  ## define our image
  # mode L is grey 8bits per pixel (range 0 to 255)
  img = Image.new("L", (glob_imgWidth, wav.getframerate() ) ) 
  print("images will be %d x %d" % img.size)
  draw = ImageDraw.Draw(img)

  ## plot to image then save
  wavLengthInSeconds = wavLengthInSeconds(wav)
  for offsetInSeconds in range(0, int(wavLengthInSeconds)):
    plotOneSecondFromWav(wav, img, draw)
    outputFilename = imageFilePrefix + "%010d" % offsetInSeconds + ".png"
    print("Writing %s" % outputFilename)
    img.save(outputFilename, "PNG")
